==> numpy_typing-1.1.0.tar.gz: numpy_typing-1.1.0/numpy_typing/content/typing.py <==
"""Numpy type hinting"""


# from t import Generic, TypeVar, overload
from typing import Generic, TypeVar, overload
from typing_extensions import Self
from numpy import ndarray

import numpy as nn



T  = TypeVar('T')
T1 = TypeVar('T1')
T2 = TypeVar('T2')
T3 = TypeVar('T3')
T4 = TypeVar('T4')


class array_1d(Generic[T, T1],ndarray):
    @overload
    def __getitem__(self, key:int) -> T:...
    @overload
    def __getitem__(self, key:"slice|list") -> Self: ...

    def __add__(self, other:"Self|int|float") -> Self: ...
    def __sub__(self, other:"Self|int|float") -> Self: ...
    def __mul__(self, other:"Self|int|float") -> Self: ...
    def __truediv__(self, other:"Self|int|float") -> Self: ...
    def __floordiv__(self, other:"Self|int|float") -> Self: ...
    def __mod__(self, other:"Self|int|float") -> Self: ...
    def __pow__(self, other:"Self|int|float") -> Self: ...

    def __eq__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __ne__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __lt__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __le__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __gt__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __ge__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...



class array_2d(Generic[T, T1, T2], ndarray):
    @overload
    def __getitem__(self, key:"tuple[int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int]") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list]|int") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list]|slice|list") -> Self: ...

    def __add__(self, other:"Self|int|float") -> Self: ...
    def __sub__(self, other:"Self|int|float") -> Self: ...
    def __mul__(self, other:"Self|int|float") -> Self: ...
    def __truediv__(self, other:"Self|int|float") -> Self: ...
    def __floordiv__(self, other:"Self|int|float") -> Self: ...
    def __mod__(self, other:"Self|int|float") -> Self: ...
    def __pow__(self, other:"Self|int|float") -> Self: ...

    def __eq__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __ne__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __lt__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __le__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __gt__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __ge__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...



class array_3d(Generic[T, T1, T2, T3], ndarray):
    @overload
    def __getitem__(self, key:"tuple[int, int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, slice|list]|tuple[int, int]") -> array_1d[T, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int]") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, slice|list]|tuple[int, slice|list]|int") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int]") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, slice|list]|tuple[slice|list, int]") -> array_2d[T, T1, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, int]|int") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, slice|list]|tuple[slice|list, slice|list]|tuple[slice|list]") -> Self: ...

    def __add__(self, other:"Self|int|float") -> Self: ...
    def __sub__(self, other:"Self|int|float") -> Self: ...
    def __mul__(self, other:"Self|int|float") -> Self: ...
    def __truediv__(self, other:"Self|int|float") -> Self: ...
    def __floordiv__(self, other:"Self|int|float") -> Self: ...
    def __mod__(self, other:"Self|int|float") -> Self: ...
    def __pow__(self, other:"Self|int|float") -> Self: ...

    def __eq__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __ne__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __lt__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __le__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __gt__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __ge__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...




class array_4d(Generic[T, T1, T2, T3, T4], ndarray):
    @overload
    def __getitem__(self, key:"tuple[int, int, int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, int, slice|list]|tuple[int, int, int]") -> array_1d[T, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, slice|list, int]") -> array_1d[T, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[int, int, slice|list, slice|list]
                             |tuple[int, int, slice|list]
                             |tuple[int, int]""") -> array_2d[T, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int, int]") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int, slice|list]|tuple[int, slice|list, int]") -> array_2d[T, T2, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, slice|list, int]") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[int, slice|list, slice|list, slice|list]
                             |tuple[int, slice|list, slice|list]
                             |tuple[int, slice|list]
                             |int""") -> array_3d[T, T2, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int, int]") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int, slice|list]|tuple[slice|list, int, int]") -> array_2d[T, T1, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, slice|list, int]") -> array_2d[T, T1, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, int, slice|list, slice|list]
                             |tuple[slice|list, int, slice|list]
                             |tuple[slice|list, int]""") -> array_3d[T, T1, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, int, int]") -> array_2d[T, T1, T2]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, slice|list, int, slice|list]
                             |tuple[slice|list, slice|list, int]""") -> array_3d[T, T1, T2, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, slice|list, int]") -> array_3d[T, T1, T2, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, slice|list, slice|list, slice|list]
                             |tuple[slice|list, slice|list, slice|list]
                             |tuple[slice|list, slice|list]
                             |tuple[slice|list]""") -> Self: ...

    def __add__(self, other:"Self|int|float") -> Self: ...
    def __sub__(self, other:"Self|int|float") -> Self: ...
    def __mul__(self, other:"Self|int|float") -> Self: ...
    def __truediv__(self, other:"Self|int|float") -> Self: ...
    def __floordiv__(self, other:"Self|int|float") -> Self: ...
    def __mod__(self, other:"Self|int|float") -> Self: ...
    def __pow__(self, other:"Self|int|float") -> Self: ...

    def __eq__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __ne__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __lt__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __le__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __gt__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __ge__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...



class float32_1d(array_1d[nn.float32, T1]): ...
class float64_1d(array_1d[nn.float64, T1]): ...
class int8_1d   (array_1d[nn.int8,    T1]): ...
class int32_1d  (array_1d[nn.int32,   T1]): ...
class int64_1d  (array_1d[nn.int64,   T1]): ...
class uint8_1d  (array_1d[nn.uint8,   T1]): ...
class uint32_1d (array_1d[nn.uint32,  T1]): ...
class uint64_1d (array_1d[nn.uint64,  T1]): ...
class bool_1d   (array_1d[bool,       T1]): ...
class str_1d    (array_1d[str,        T1]): ...

class float32_2d(array_2d[nn.float32, T1, T2]): ...
class float64_2d(array_2d[nn.float64, T1, T2]): ...
class int8_2d   (array_2d[nn.int8,    T1, T2]): ...
class int32_2d  (array_2d[nn.int32,   T1, T2]): ...
class int64_2d  (array_2d[nn.int64,   T1, T2]): ...
class uint8_2d  (array_2d[nn.uint8,   T1, T2]): ...
class uint32_2d (array_2d[nn.uint32,  T1, T2]): ...
class uint64_2d (array_2d[nn.uint64,  T1, T2]): ...
class bool_2d   (array_2d[bool,       T1, T2]): ...
class str_2d    (array_2d[str,        T1, T2]): ...

class float32_3d(array_3d[nn.float32, T1, T2, T3]): ...
class float64_3d(array_3d[nn.float64, T1, T2, T3]): ...
class int8_3d   (array_3d[nn.int8,    T1, T2, T3]): ...
class int32_3d  (array_3d[nn.int32,   T1, T2, T3]): ...
class int64_3d  (array_3d[nn.int64,   T1, T2, T3]): ...
class uint8_3d  (array_3d[nn.uint8,   T1, T2, T3]): ...
class uint32_3d (array_3d[nn.uint32,  T1, T2, T3]): ...
class uint64_3d (array_3d[nn.uint64,  T1, T2, T3]): ...
class bool_3d   (array_3d[bool,       T1, T2, T3]): ...
class str_3d    (array_3d[str,        T1, T2, T3]): ...

class float32_4d(array_4d[nn.float32, T1, T2, T3, T4]): ...
class float64_4d(array_4d[nn.float64, T1, T2, T3, T4]): ...
class int8_4d   (array_4d[nn.int8,    T1, T2, T3, T4]): ...
class int32_4d  (array_4d[nn.int32,   T1, T2, T3, T4]): ...
class int64_4d  (array_4d[nn.int64,   T1, T2, T3, T4]): ...
class uint8_4d  (array_4d[nn.uint8,   T1, T2, T3, T4]): ...
class uint32_4d (array_4d[nn.uint32,  T1, T2, T3, T4]): ...
class uint64_4d (array_4d[nn.uint64,  T1, T2, T3, T4]): ...
class bool_4d   (array_4d[bool,       T1, T2, T3, T4]): ...
class str_4d    (array_4d[str,        T1, T2, T3, T4]): ...

==> numpy_typing-1.1.1.tar.gz: numpy_typing-1.1.1/numpy_typing/content/typing.py <==
"""Numpy type hinting"""


# from t import Generic, TypeVar, overload
from typing import Generic, TypeVar, overload
from typing_extensions import Self, Any
from numpy import ndarray
from . import axis as ax
import numpy as nn



T  = TypeVar('T')
T1 = TypeVar('T1')
T2 = TypeVar('T2')
T3 = TypeVar('T3')
T4 = TypeVar('T4')


class array(ndarray):
    axis: "dict[ax.__axis__, int]" = {}
    batch: ax.batch = None
    sample: ax.sample = None
    values: ax.values = None
    time: ax.time = None
    feature: ax.feature = None
    item: ax.item = None
    x: ax.x = None
    y: ax.y = None
    z: ax.z = None
    rgb: ax.rgb = None
    rgba: ax.rgba = None
    label: ax.label = None




    def __add__(self, other:"Self|int|float") -> Self: ...
    def __sub__(self, other:"Self|int|float") -> Self: ...
    def __mul__(self, other:"Self|int|float") -> Self: ...
    def __truediv__(self, other:"Self|int|float") -> Self: ...
    def __floordiv__(self, other:"Self|int|float") -> Self: ...
    def __mod__(self, other:"Self|int|float") -> Self: ...
    def __pow__(self, other:"Self|int|float") -> Self: ...

    def __str__(self) -> str:
        return nn.array(self, copy=False).__str__()


class array_1d(Generic[T, T1],array):

    @overload
    def __getitem__(self, key:int) -> T:...
    @overload
    def __getitem__(self, key:"slice|list") -> Self: ...


    def __eq__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __ne__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __lt__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __le__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __gt__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...
    def __ge__(self, other:"Self|int|float") -> "array_1d[bool, T1]": ...




class array_2d(Generic[T, T1, T2], array):
    @overload
    def __getitem__(self, key:"tuple[int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int]|T1") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list]|int|T2") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list]|slice|list") -> Self: ...


    def __eq__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __ne__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __lt__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __le__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __gt__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...
    def __ge__(self, other:"Self|int|float") -> "array_2d[bool, T1, T2]": ...

    @overload
    def sum(self, axis:None) -> T: ...
    @overload
    def sum(self, axis:T1) -> array_1d[T, T2]: ...
    @overload
    def sum(self, axis:T2) -> array_1d[T, T1]: ...
    @overload
    def sum(self, axis:"tuple[T1, T2]") -> T: ...

    def sum(self, axis:ax.__axis__=None):
        if (isinstance(axis, ax.__axis__)):
            res = super().sum(axis.i)
            new_axis = [ax for ax in self.axis if ax.tag != axis.tag]
        elif (isinstance(axis, int)):
            res = super().sum(axis)
            new_axis = [ax for ax in self.axis if ax.i != axis]
        elif (axis == None):
            return super().sum(axis)


        res=___init___(res, new_axis)
        print(axis, self.axis)
        return res




class array_3d(Generic[T, T1, T2, T3], array):
    @overload
    def __getitem__(self, key:"tuple[int, int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, slice|list]|tuple[int, int]") -> array_1d[T, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int]") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, slice|list]|tuple[int, slice|list]|int") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int]") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, slice|list]|tuple[slice|list, int]") -> array_2d[T, T1, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, int]|int") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, slice|list]|tuple[slice|list, slice|list]|tuple[slice|list]") -> Self: ...


    def __eq__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __ne__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __lt__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __le__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __gt__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...
    def __ge__(self, other:"Self|int|float") -> "array_3d[bool, T1, T2, T3]": ...



class array_4d(Generic[T, T1, T2, T3, T4], array):
    @overload
    def __getitem__(self, key:"tuple[int, int, int, int]") -> T: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, int, slice|list]|tuple[int, int, int]") -> array_1d[T, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, int, slice|list, int]") -> array_1d[T, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[int, int, slice|list, slice|list]
                             |tuple[int, int, slice|list]
                             |tuple[int, int]""") -> array_2d[T, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int, int]") -> array_1d[T, T2]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, int, slice|list]|tuple[int, slice|list, int]") -> array_2d[T, T2, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[int, slice|list, slice|list, int]") -> array_2d[T, T2, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[int, slice|list, slice|list, slice|list]
                             |tuple[int, slice|list, slice|list]
                             |tuple[int, slice|list]
                             |int""") -> array_3d[T, T2, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int, int]") -> array_1d[T, T1]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, int, slice|list]|tuple[slice|list, int, int]") -> array_2d[T, T1, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, int, slice|list, int]") -> array_2d[T, T1, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, int, slice|list, slice|list]
                             |tuple[slice|list, int, slice|list]
                             |tuple[slice|list, int]""") -> array_3d[T, T1, T3, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, int, int]") -> array_2d[T, T1, T2]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, slice|list, int, slice|list]
                             |tuple[slice|list, slice|list, int]""") -> array_3d[T, T1, T2, T4]: ...
    @overload
    def __getitem__(self, key:"tuple[slice|list, slice|list, slice|list, int]") -> array_3d[T, T1, T2, T3]: ...
    @overload
    def __getitem__(self, key:"""tuple[slice|list, slice|list, slice|list, slice|list]
                             |tuple[slice|list, slice|list, slice|list]
                             |tuple[slice|list, slice|list]
                             |tuple[slice|list]""") -> Self: ...

    def __eq__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __ne__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __lt__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __le__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __gt__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...
    def __ge__(self, other:"Self|int|float") -> "array_4d[bool, T1, T2, T3, T4]": ...



class float32_1d(array_1d[nn.float32, T1]): ...
class float64_1d(array_1d[nn.float64, T1]): ...
class int8_1d   (array_1d[nn.int8,    T1]): ...
class int32_1d  (array_1d[nn.int32,   T1]): ...
class int64_1d  (array_1d[nn.int64,   T1]): ...
class uint8_1d  (array_1d[nn.uint8,   T1]): ...
class uint32_1d (array_1d[nn.uint32,  T1]): ...
class uint64_1d (array_1d[nn.uint64,  T1]): ...
class bool_1d   (array_1d[bool,       T1]): ...
class str_1d    (array_1d[str,        T1]): ...

class float32_2d(array_2d[nn.float32, T1, T2]): ...
class float64_2d(array_2d[nn.float64, T1, T2]): ...
class int8_2d   (array_2d[nn.int8,    T1, T2]): ...
class int32_2d  (array_2d[nn.int32,   T1, T2]): ...
class int64_2d  (array_2d[nn.int64,   T1, T2]): ...
class uint8_2d  (array_2d[nn.uint8,   T1, T2]): ...
class uint32_2d (array_2d[nn.uint32,  T1, T2]): ...
class uint64_2d (array_2d[nn.uint64,  T1, T2]): ...
class bool_2d   (array_2d[bool,       T1, T2]): ...
class str_2d    (array_2d[str,        T1, T2]): ...

class float32_3d(array_3d[nn.float32, T1, T2, T3]): ...
class float64_3d(array_3d[nn.float64, T1, T2, T3]): ...
class int8_3d   (array_3d[nn.int8,    T1, T2, T3]): ...
class int32_3d  (array_3d[nn.int32,   T1, T2, T3]): ...
class int64_3d  (array_3d[nn.int64,   T1, T2, T3]): ...
class uint8_3d  (array_3d[nn.uint8,   T1, T2, T3]): ...
class uint32_3d (array_3d[nn.uint32,  T1, T2, T3]): ...
class uint64_3d (array_3d[nn.uint64,  T1, T2, T3]): ...
class bool_3d   (array_3d[bool,       T1, T2, T3]): ...
class str_3d    (array_3d[str,        T1, T2, T3]): ...

class float32_4d(array_4d[nn.float32, T1, T2, T3, T4]): ...
class float64_4d(array_4d[nn.float64, T1, T2, T3, T4]): ...
class int8_4d   (array_4d[nn.int8,    T1, T2, T3, T4]): ...
class int32_4d  (array_4d[nn.int32,   T1, T2, T3, T4]): ...
class int64_4d  (array_4d[nn.int64,   T1, T2, T3, T4]): ...
class uint8_4d  (array_4d[nn.uint8,   T1, T2, T3, T4]): ...
class uint32_4d (array_4d[nn.uint32,  T1, T2, T3, T4]): ...
class uint64_4d (array_4d[nn.uint64,  T1, T2, T3, T4]): ...
class bool_4d   (array_4d[bool,       T1, T2, T3, T4]): ...
class str_4d    (array_4d[str,        T1, T2, T3, T4]): ...



def ___init___(np_array:ndarray, axis:"list[ax.__axis__]") -> None:
    if (np_array.ndim == 1):
        np_array = array_1d(np_array.shape, np_array.dtype, buffer=np_array)
    elif (np_array.ndim == 2):
        np_array = array_2d(np_array.shape, np_array.dtype, buffer=np_array)
    elif (np_array.ndim == 3):
        np_array = array_3d(np_array.shape, np_array.dtype, buffer=np_array)
    elif (np_array.ndim == 4):
        np_array = array_4d(np_array.shape, np_array.dtype, buffer=np_array)

    if (np_array.ndim <= 4):
        array.axis = {}
        for i in range(len(axis)):
            if (isinstance(axis[i], ax.__axis__)):
                print("keep")
                axe = axis[i].copy(i)
            else:
                print("cpy")
                axe = axis[i](i)
            array.axis[axe] = i
            setattr(array, axis[i].name, axe)

    return np_array
